import discord
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List
from discord import Guild

from bot_graphql.abc.query_abc import QueryABC


class GuildQuery(QueryABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
    ):
        QueryABC.__init__(self, "Guild")

        self._bot = bot

        self.set_field("id", lambda g, *_: g.id)
        self.set_field("name", lambda g, *_: g.name)
        self.set_field("channels", self._resolve_channels)
        self.set_field("roles", lambda g, *_: g.roles.order_by(lambda x: x.position))
        self.set_field(
            "emojis",
            lambda g, *_: List(discord.Emoji, g.emojis).order_by_descending(lambda x: x.created_at),
        )

    def _resolve_channels(self, g: Guild, *_, filter=None):
        channels = List(any).extend(g.channels).order_by(lambda x: x.position)

        if filter is None:
            return channels

        if "id" in filter:
            channels = channels.where(lambda c: c.id == int(filter["id"]))

        if "name" in filter:
            channels = channels.where(lambda c: c.id == filter["name"])

        if "type" in filter:
            channels = channels.where(lambda c: type(c).__name__ == filter["type"])

        return channels
