from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_discord.discord_event_types_enum import DiscordEventTypesEnum
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC

from bot_core.abc.module_abc import ModuleABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from modules.achievements.achievement_attribute_resolver import (
    AchievementAttributeResolver,
)
from modules.achievements.achievement_service import AchievementService
from modules.achievements.commands.achievements_group import AchievementGroup
from modules.achievements.events.achievement_on_message_event import (
    AchievementOnMessageEvent,
)
from modules.achievements.events.achievement_on_reaction_add_event import (
    AchievementOnReactionAddEvent,
)
from modules.achievements.events.achievement_on_reaction_remove_event import (
    AchievementOnReactionRemoveEvent,
)
from modules.achievements.events.achievement_on_voice_state_update_event import (
    AchievementOnVoiceStateUpdateEvent,
)


class AchievementsModule(ModuleABC):
    def __init__(self, dc: DiscordCollectionABC):
        ModuleABC.__init__(self, dc, FeatureFlagsEnum.achievements_module)

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        pass

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_transient(AchievementAttributeResolver)
        services.add_transient(AchievementService)

        services.add_transient(AchievementGroup)

        services.add_transient(DiscordEventTypesEnum.on_message.value, AchievementOnMessageEvent)
        services.add_transient(DiscordEventTypesEnum.on_reaction_add.value, AchievementOnReactionAddEvent)
        services.add_transient(
            DiscordEventTypesEnum.on_reaction_remove.value,
            AchievementOnReactionRemoveEvent,
        )
        services.add_transient(
            DiscordEventTypesEnum.on_voice_state_update.value,
            AchievementOnVoiceStateUpdateEvent,
        )
