import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.server_config_repository_abc import ServerConfigRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.technician_config_repository_abc import TechnicianConfigRepositoryABC
from bot_data.model.team_member_type_enum import TeamMemberTypeEnum
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class PermissionService(PermissionServiceABC):
    def __init__(
        self,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        config: ConfigurationABC,
        servers: ServerRepositoryABC,
        server_configs: ServerConfigRepositoryABC,
        technician_configs: TechnicianConfigRepositoryABC,
    ):
        PermissionServiceABC.__init__(self)
        self._logger = logger
        self._bot = bot
        self._config = config
        self._servers = servers
        self._server_configs = server_configs
        self._technician_configs = technician_configs

    def _has_member_role(self, member: discord.Member, team_member_type: TeamMemberTypeEnum) -> bool:
        if member is None or member.guild is None:
            return False

        self._logger.debug(__name__, f"Checking is member {member.name} {team_member_type.value}")

        try:
            server = self._servers.get_server_by_discord_id(member.guild.id)
            config = self._server_configs.get_server_config_by_server(server.id)
            roles = config.team_role_ids.where(lambda x: x.team_member_type == team_member_type).select(
                lambda x: member.guild.get_role(x.role_id)
            )
            for role in roles:
                if role not in member.roles:
                    continue

                return True
        except Exception as e:
            self._logger.error(__name__, "Permission check failed", e)

        return False

    def is_member_admin(self, member: discord.Member) -> bool:
        return self._has_member_role(member, TeamMemberTypeEnum.admin)

    def is_member_moderator(self, member: discord.Member) -> bool:
        return self._has_member_role(member, TeamMemberTypeEnum.moderator) or self._has_member_role(
            member, TeamMemberTypeEnum.admin
        )

    def is_member_technician(self, member: discord.Member) -> bool:
        if member is None or member.guild is None:
            return False

        self._logger.debug(__name__, f"Checking is member {member.name} technician")

        try:
            tech_config = self._technician_configs.get_technician_config()
            if member.id in tech_config.technician_ids:
                return True
        except Exception as e:
            self._logger.error(__name__, "Permission check failed", e)

        return False
