from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC

from bot_core.abc.module_abc import ModuleABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_data.abc.data_seeder_abc import DataSeederABC
from modules.technician.api_key_seeder import ApiKeySeeder
from modules.technician.command.api_key_group import ApiKeyGroup
from modules.technician.command.log_command import LogCommand
from modules.technician.command.restart_command import RestartCommand
from modules.technician.command.shutdown_command import ShutdownCommand
from modules.technician.command.sync_xp_command import SyncXpGroup


class TechnicianModule(ModuleABC):
    def __init__(self, dc: DiscordCollectionABC):
        ModuleABC.__init__(self, dc, FeatureFlagsEnum.base_module)

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        pass

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_transient(DataSeederABC, ApiKeySeeder)
        # commands
        services.add_transient(RestartCommand)
        services.add_transient(ShutdownCommand)
        services.add_transient(LogCommand)
        services.add_transient(ApiKeyGroup)
        services.add_transient(SyncXpGroup)
        # events
