import { Component, EventEmitter, Input, Output, ViewChild } from "@angular/core";
import { ToastService } from "../../../../services/toast/toast.service";
import { TranslateService } from "@ngx-translate/core";
import { elementAt } from "rxjs";

interface UploadEvent {
  originalEvent: Event;
  files: File[];
}

@Component({
  selector: "app-data-import-and-export",
  templateUrl: "./data-import-and-export.component.html",
  styleUrls: ["./data-import-and-export.component.scss"]
})
export class DataImportAndExportComponent {

  @ViewChild("upload") upload: any;

  private _data: any[] = [];

  @Input() name: string = "";

  @Input()
  set data(data: any[]) {
    this._data = data;
    this.dataChange.emit(data);
  }

  get data(): any[] {
    return this._data;
  }

  @Output() dataChange: EventEmitter<any[]> = new EventEmitter<any[]>();
  @Input() callback!: (data: any[], isNew: boolean) => void;
  @Input() validator: (oldElement: any, newElement: any) => boolean = (oldElement: any, newElement: any) => {
    return true;
  };

  public constructor(
    private toastService: ToastService,
    private translate: TranslateService
  ) {

  }

  public export() {
    const json = JSON.stringify(this.data);
    const element = document.createElement("a");
    element.setAttribute("href", "data:application/json;charset=UTF-8," + encodeURIComponent(json));
    element.setAttribute("download", `${this.name}.json`);
    element.style.display = "none";
    document.body.appendChild(element);
    element.click(); // simulate click
    document.body.removeChild(element);
  }

  public import(event: { files: File[] }) {
    const file = event.files[0];
    const fileReader = new FileReader();
    fileReader.onload = () => {
      if (!fileReader.result) return;
      const newData: any[] = JSON.parse(fileReader.result.toString());
      this.upload.clear();
      newData.forEach(element => {
        element.id = 0;
      });
      this.data.forEach(element => {
        const existingElement = newData.find(x => this.validator(x, element));
        if (existingElement) {
          const index = this.data.indexOf(element);
          const oldId = element.id;
          element = existingElement;
          element.id = oldId;
          this.data[index] = element;
          newData.splice(newData.indexOf(existingElement), 1);
        }
      });
      this.callback(this.data, false);
      this.callback(newData, true);
      this.data.push(...newData);
      this.toastService.success(this.translate.instant("common.file.uploaded"), this.translate.instant("common.file.uploaded"));
    };
    fileReader.readAsText(file, "UTF-8");
  }
}
